import 'dart:convert';

import 'package:equatable/equatable.dart';
import 'package:json_annotation/json_annotation.dart';

part 'story.g.dart';

class StoryType {
  static String get image => "image";
  static String get video => "video";
  static String get quiz => "quiz";
  static String get quizList => "quizList";
}

class Story extends Equatable {
  final String id;
  final String thumbnail;
  final StoryUser user;
  final DateTime recordedAt;
  final StoryBackground background;
  final StoryData data;
  final List<String> seenBy;

  Story(this.id, this.thumbnail, this.user, this.recordedAt, this.background,
      this.data, this.seenBy);

  Story.named(
      {this.id,
      this.thumbnail,
      this.user,
      this.recordedAt,
      this.background,
      this.data,
      this.seenBy});

  copyWith({String id, data, recordedAt, thumbnail, user, seenBy}) {
    return Story.named(
        id: id ?? this.id,
        data: data ?? this.data,
        background: background ?? this.background,
        recordedAt: recordedAt ?? this.recordedAt,
        thumbnail: thumbnail ?? this.thumbnail,
        user: user ?? this.user,
        seenBy: seenBy ?? this.seenBy);
  }

  @override
  List<Object> get props => [id];

  Map<String, dynamic> toJson() {
    return {
      "id": id,
      "background": background is StoryImage
          ? jsonEncode((background as StoryImage).toJson())
          : background is StoryVideo
              ? jsonEncode((background as StoryVideo).toJson())
              : jsonEncode({}),
      'data': jsonEncode({}),
      'recordedAt': recordedAt?.toIso8601String(),
      'user': user?.toJson(),
      'thumbnail': thumbnail,
      'seenBy': seenBy ?? []
    };
  }

  factory Story.fromJson(json) {
    return Story.named(
        id: json["id"],
        background: json["background"] != null
            ? json["background"]["type"] == StoryType.image
                ? StoryImage.fromJson(json["background"])
                : json["background"]["type"] == StoryType.video
                    ? StoryVideo.fromJson(json["background"])
                    : null
            : null,
        recordedAt: DateTime.tryParse("${json["recordedAt"]}"),
        seenBy: List<String>.from(json["seenBy"] ?? []),
        thumbnail: json["thumbnail"],
        user: json["user"] != null ? StoryUser.fromJson(json["user"]) : null);
  }
}

@JsonSerializable(explicitToJson: true)
class StoryUser {
  final String id;
  final String name;
  final String profilePic;

  StoryUser(this.id, this.name, this.profilePic);
  StoryUser.named({this.id, this.name, this.profilePic});

  Map<String, dynamic> toJson() => _$StoryUserToJson(this);

  factory StoryUser.fromJson(json) => _$StoryUserFromJson(json);
}

abstract class StoryBackground {
  final String type;

  StoryBackground(
    this.type,
  );
}

@JsonSerializable(explicitToJson: true)
class StoryVideo extends StoryBackground {
  final String url;
  final String thumbnail;

  StoryVideo(this.url, this.thumbnail) : super(StoryType.video);

  StoryVideo.named({this.url, this.thumbnail}) : super(StoryType.video);

  Map<String, dynamic> toJson() => _$StoryVideoToJson(this);

  factory StoryVideo.fromJson(json) => _$StoryVideoFromJson(json);
}

@JsonSerializable(explicitToJson: true)
class StoryImage extends StoryBackground {
  final String url;

  StoryImage(this.url) : super(StoryType.image);

  StoryImage.named({this.url}) : super(StoryType.image);

  Map<String, dynamic> toJson() => _$StoryImageToJson(this);

  factory StoryImage.fromJson(json) => _$StoryImageFromJson(json);
}

abstract class StoryData {
  final String type;

  StoryData(
    this.type,
  );
}
